'use strict';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorHandler = void 0;
const youch_1 = __importDefault(require("youch"));
const http_error_1 = require("./http-error");
const goodies_1 = require("@supercharge/goodies");
const collections_1 = __importDefault(require("@supercharge/collections"));
class ErrorHandler {
    /**
     * Create a new error handler instance.
     */
    constructor(app) {
        this.app = app;
        this.ignoredErrors = [];
        this.reportCallbacks = [];
        this.register();
    }
    /**
     * The application’s logger instance.
     */
    logger() {
        return this.app.logger();
    }
    /**
     * The application’s view renderer.
     */
    view() {
        return this.app.make('view');
    }
    /**
     * Register the error handling callbacks. For example, to report error
     * upstream to an error tracking service, like Sentry or Bugsnag.
     */
    register() {
        //
    }
    /**
     * Tell the error handler to not report the `error` type.
     */
    ignore(error) {
        return (0, goodies_1.tap)(this, () => {
            this.ignoredErrors.push(error);
        });
    }
    /**
     * Returns an array of errors that should not be reported.
     */
    dontReport() {
        return [];
    }
    /**
     * Determine whether to report the given `error`.
     */
    shouldntReport(error) {
        return this.dontReport().concat(this.ignoredErrors).some(ErrorConstructor => {
            return error instanceof ErrorConstructor;
        });
    }
    /**
     * Register a reportable callback.
     *
     * @param  {Function} reportUsing
     *
     * @returns {ErrorHandler}
     */
    reportable(reportUsing) {
        return (0, goodies_1.tap)(this, () => {
            this.reportCallbacks.push(reportUsing);
        });
    }
    /**
     * Returns the error’s context for logging.
     *
     * @param {*}error
     *
     * @returns {Record<string, any>}
     */
    errorContext(error) {
        if (typeof error.context === 'function') {
            return { ...error.context() };
        }
        return {};
    }
    /**
     * Returns the default logging context.
     *
     * @param {HttpContext} ctx
     *
     * @returns {*}
     */
    context(_ctx) {
        return {};
    }
    /**
     * Handle the given error.
     *
     * @param {HttpContext} ctx
     * @param {Error} error
     */
    async handle(ctx, error) {
        await this.report(ctx, error);
        await this.render(ctx, error);
    }
    /**
     * Report an error.
     *
     * @param {HttpContext} ctx
     * @param {HttpError} error
     */
    async report(ctx, error) {
        if (this.shouldntReport(error)) {
            return;
        }
        if (await this.errorReported(ctx, error)) {
            return;
        }
        const handled = await (0, collections_1.default)(this.reportCallbacks).any(async (reportCallback) => {
            return await reportCallback(ctx, error);
        });
        if (handled) {
            return;
        }
        const context = {
            ...this.context(ctx),
            ...this.errorContext(error),
            error,
        };
        this.logger().error(error.message, context);
    }
    /**
     * Determine whether the given `error` is implementing a `handle` method and
     * that `handle` method returns a truthy value, like a valid HTTP response.
     *
     * @param {HttpContext} ctx
     * @param {Error} error
     *
     * @returns {*}
     */
    async errorReported(ctx, error) {
        if (typeof error.report !== 'function') {
            return false;
        }
        return await error.report(error, ctx);
    }
    /**
     * Render the error into an HTTP response.
     *
     * @param {HttpContext} ctx
     * @param {HttpError} error
     */
    async render(ctx, error) {
        if (await this.errorRendered(ctx, error)) {
            return;
        }
        const httpError = http_error_1.HttpError.wrap(error);
        if (ctx.request.wantsJson()) {
            return this.renderJsonResponse(ctx, httpError);
        }
        if (this.app.env().isProduction()) {
            return this.renderJsonResponse(ctx, httpError);
        }
        return await this.renderViewResponse(ctx, httpError);
    }
    /**
     * Determine whether the given `error` is implementing a `handle` method and
     * that `handle` method returns a truthy value, like a valid HTTP response.
     *
     * @param {HttpContext} ctx
     * @param {Error} error
     *
     * @returns {*}
     */
    async errorRendered(ctx, error) {
        if (typeof error.render !== 'function') {
            return false;
        }
        return await error.render(ctx, error);
    }
    /**
     * Creates a JSON response depending on the app’s environment.
     *
     * @param {HttpContext} ctx
     * @param {HttpError} error
     */
    renderJsonResponse(ctx, error) {
        const { message, stack, status: statusCode } = error;
        this.app.env().isProduction()
            ? ctx.response.status(statusCode).payload({ message, statusCode })
            : ctx.response.status(statusCode).payload({ message, statusCode, stack });
    }
    /**
     * Creates an HTML response depending on the app’s environment.
     *
     * @param {HttpContext} ctx
     * @param {HttpError} error
     */
    async renderViewResponse(ctx, error) {
        if (await this.isMissingTemplateFor(error)) {
            return await this.renderYouchResponse(ctx, error);
        }
        await ctx.response.status(error.status).view(this.viewTemplateFor(error), { error });
    }
    /**
     * Determine whether a view template file is missing for the given `error`.
     *
     * @param {HttpError} error
     *
     * @returns {Boolean}
     */
    async isMissingTemplateFor(error) {
        return !await this.templateExistsFor(error);
    }
    /**
     * Determine whether a view template file exists for the given `error`.
     *
     * @param {HttpError} error
     *
     * @returns {Boolean}
     */
    async templateExistsFor(error) {
        return await this.view().exists(this.viewTemplateFor(error));
    }
    /**
     * Returns the view template file for the given `error`.
     *
     * @param {HttpError} error
     *
     * @returns {String}
     */
    viewTemplateFor(error) {
        return `errors/${error.status}`;
    }
    /**
     * Renders an HTML response containing details about the given `error`.
     *
     * @param {HttpContext} ctx
     * @param {HttpError} error
     */
    async renderYouchResponse({ request, response }, error) {
        response.payload(await new youch_1.default(error, request.req()).toHTML()).status(error.status);
    }
}
exports.ErrorHandler = ErrorHandler;
