'use strict';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ShutdownSignalListener = void 0;
const set_1 = __importDefault(require("@supercharge/set"));
class ShutdownSignalListener {
    /**
     * Create a new instance.
     */
    constructor(app) {
        /**
         * Invoke callback and exit process
         */
        this.kill = async function () {
            try {
                await Promise.race([
                    this.onShutdownCallback(),
                    new Promise((resolve) => {
                        setTimeout(resolve, 3000);
                    }),
                ]);
                process.exit(0);
            }
            catch (error) {
                process.exit(1);
            }
        }.bind(this);
        this.app = app;
        this.onShutdownCallback = () => { };
        this.shutdownSignals = ['SIGINT', 'SIGTERM'];
    }
    /**
     * Assign the given shutdown `callback` that runs when receiving a shutdown signal.
     */
    onShutdown(callback) {
        this.onShutdownCallback = callback;
        return this;
    }
    /**
     * Close on SIGINT AND SIGTERM SIGNALS
     */
    listen(...signals) {
        this.runShutdownCallbackOn(...signals);
        /**
         * Log the uncaught exception error.
         */
        process.on('uncaughtExceptionMonitor', (error) => {
            this.app.logger().alert(error.message, { error });
        });
    }
    /**
     * Register the shutdown handler to run for the given `signals`.
     */
    runShutdownCallbackOn(...signals) {
        this.shutdownSignals = set_1.default.from(...signals).concat(this.shutdownSignals).toArray();
        this.shutdownSignals.forEach(signal => {
            // eslint-disable-next-line @typescript-eslint/no-misused-promises
            process.on(signal, this.kill);
        });
    }
    /**
     * Remove the event listeners for shutdown signals and
     * reset the shutdown callback to a no-op function.
     */
    cleanup() {
        this.shutdownSignals.forEach(signal => {
            // eslint-disable-next-line @typescript-eslint/no-misused-promises
            process.removeListener(signal, this.kill);
        });
        this.onShutdownCallback = () => { };
        return this;
    }
}
exports.ShutdownSignalListener = ShutdownSignalListener;
